<?php
	require_once('sql.class.php');
	/**
	* Board class. Meant for interfacing with an SMF board.
	*
	* @module smf
	* @class Board
	* @constructor
	*/
	class Board implements JsonSerializable{
		/**
		* The board ID
		* 
		* @property id
		* @type {Integer}
		* @public
		* @required
		*/
		public $id;
		/**
		* The board name
		* 
		* @property name
		* @type {String}
		* @public
		* @required
		*/
		public $name;
		/**
		* The id of the category the board belongs to
		* 
		* @property id_cat
		* @type {Integer}
		* @public
		* @required
		*/
		public $id_cat;
		/**
		* The id of the parent board
		* 
		* @property id_parent
		* @type {Integer}
		* @public
		* @required
		*/
		public $id_parent;
		/**
		* A count of all the topics in the board
		* 
		* @property topics
		* @type {Integer}
		* @public
		* @required
		*/
		public $topics_count;
		/**
		* A count of all the posts on the board
		* 
		* @property posts
		* @type {Integer}
		* @public
		* @required
		*/
		public $posts_count;
		public $member_groups;
		/**
		* A reference to the parent SMF object
		* 
		* @property smf
		* @type {SMF}
		* @private
		* @required
		*/
		public $smf;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->id = $id;
			$this->smf = $smf;
			if($id == 0){
				$this->id_cat = 0;
				$this->id_parent = 0;
				$this->name = '[ROOT]';
				$this->topics_count = 0;
				$this->posts_count = 0;
			}else{
				$b = $smf->sql->query("
					select	id_cat,
							id_parent,
							name,
							num_topics,
							num_posts,
							member_groups
					from {$db_prefix}boards b
					WHERE id_board = ?;
				",'i',$this->id)->assoc_result;
				if($b){
					$this->id_cat = $b['id_cat'];
					$this->id_parent = $b['id_parent'];
					$this->name = $b['name'];
					$this->topics_count = $b['num_topics'];
					$this->posts_count = $b['num_posts'];
					$this->member_groups = $b['member_groups'];
				}
			}
		}
		public function jsonSerialize(){
			$children = array();
			foreach($this->children as $board){
				array_push($children,array(
					'id'=>intval($board->id),
					'name'=>utf8_encode($board->name)
				));
			}
			return array(
				'id'=>intval($this->id),
				'name'=>utf8_encode($this->name),
				'colour'=>$this->colour,
				'stats'=>array(
					'posts'=>$this->posts_count,
					'topics'=>$this->topics_count,
					'total_posts'=>$this->total_posts,
					'total_topics'=>$this->total_topics
				),
				'category'=>array(
					'id'=>intval($this->category->id),
					'name'=>$this->category->name
				),
				'parent'=>array(
					'id'=>intval($this->parent->id),
					'name'=>$this->parent->name
				),
				'children'=>$children
			);
		}
		public function __toString(){
			return "[Board {$this->id}] {\n  name: ".strip_tags($this->name)."\n  posts: {$this->posts_count}\n  topics: {$this->topics_count}\n  total_posts: {$this->total_posts}\n  total_topics: {$this->total_topics}\n}";
		}
		/**
		* Returns the child board specified by the iterator
		*
		* @method get
		* @param {Integer} id The iterator for the child board
		* @return {Board} Returns the child board
		*/
		public function get($id){
			return $this->children[$id];
		}
		public function __get($name){
			global $db_prefix;
			switch($name){
				/**
				* An array of child boards.
				* 
				* @property children
				* @type {Array}
				* @public
				*/
				case 'children':
					$boards = $this->smf->sql->query("
						select	id_board as id
						from {$db_prefix}boards b
						WHERE id_parent = ?;
					",'i',$this->id)->assoc_results;
					$c = array();
					foreach($boards as $board){
						array_push($c,new Board($board['id'],$this->smf));
					}
					return $c;
				break;
				/**
				* The parent board
				* 
				* @property parent
				* @type {Board}
				* @public
				*/
				case 'parent':
					return new Board($this->id_parent,$this->smf);
				break;
				/**
				* The total posts this board, and all child boards have
				* 
				* @property total_posts
				* @type {Integer}
				* @public
				*/
				case 'total_posts':
					$total = $this->posts_count;
					foreach($this->children as $child){
						$total += $child->total_posts;
					}
					return $total;
				break;
				/**
				* The total topics this board and all child boards have
				* 
				* @property total_topics
				* @type {Integer}
				* @public
				*/
				case 'total_topics':
					$total = $this->topics_count;
					foreach($this->children as $child){
						$total += $child->total_topics;
					}
					return $total;
				break;
				/**
				* Returns the maximum total topics of all the child boards
				* 
				* @property max_total_topics
				* @type {Integer}
				* @public
				*/
				case 'max_total_topics':
					$max = 0;
					foreach($this->children as $child){
						if($child->total_topics > $max){
							$max = $child->total_topics;
						}
					}
					return $max;
				break;
				/**
				* Returns the maximum total posts of all the child boards
				* 
				* @property max_total_posts
				* @type {Integer}
				* @public
				*/
				case 'max_total_posts':
					$max = 0;
					foreach($this->children as $child){
						if($child->total_posts > $max){
							$max = $child->total_posts;
						}
					}
					return $max;
				break;
				/**
				* Returns the maximum topics of all the child boards and this board
				* 
				* @property max_topics
				* @type {Integer}
				* @public
				*/
				case 'max_topics':
					$max = $this->topics_count;
					foreach($this->children as $child){
						if($child->max_topics > $max){
							$max = $child->max_topics;
						}
					}
					return $max;
				break;
				/**
				* Returns the maximum posts of all the child boards and this board
				* 
				* @property max_posts
				* @type {Integer}
				* @public
				*/
				case 'max_posts':
					$max = $this->posts_count;
					foreach($this->children as $child){
						if($child->max_posts > $max){
							$max = $child->max_posts;
						}
					}
					return $max;
				break;
				/**
				* Returns the maximum size of the name of any child board
				* 
				* @property max_name_all
				* @type {Integer}
				* @public
				*/
				case 'max_name_all':
					$max = 0;
					foreach($this->children as $child){
						if(strlen($child->name)>$max){
							$max = strlen($child->name);
						}
					}
					return $max;
				break;
				/**
				* Returns the maximum size of the name of any child board or this board
				* 
				* @property max_name
				* @type {Integer}
				* @public
				*/
				case 'max_name':
					$max = strlen($this->name);
					foreach($this->children as $child){
						if($child->max_name>$max){
							$max = $child->max_name;
						}
					}
					return $max;
				break;
				/**
				* The colour to use for this board. Based on the name
				* 
				* @property colour
				* @type {Array(String,Integer,Integer,Integer)}
				* @public
				*/
				case 'colour':
					$code = substr(dechex(crc32($this->name)),0,6);
					return array(
						'#'.$code,
						hexdec(substr($code,0,2)),
						hexdec(substr($code,2,2)),
						hexdec(substr($code,4))
					);
				break;
				/**
				* Returns the count of how many children this board has
				* 
				* @property count
				* @type {Integer}
				* @public
				*/
				case 'count':
					$boards = $this->smf->sql->query("
						select	count(id_board) as count
						from {$db_prefix}boards b
						WHERE id_parent = ?;
					",'i',$this->id)->assoc_result;
					return $boards?$boards['count']:0;
				break;
				/**
				* Retuns the category this board is in.
				* 
				* @property category
				* @type {Category}
				* @public
				*/
				case 'category':
					return new Category($this->id_cat,$this->smf);
				break;
				case 'topics':
					$topics = $this->smf->sql->query("
						SELECT	id_topic as id
						FROM {$db_prefix}topics t
						WHERE t.id_board = ?;
					",'i',$this->id)->assoc_results;
					$a = array();
					foreach($topics as $topic){
						array_push($a,$this->smf->topic($topic['id']));
					}
					return $a;
				break;
				case 'first_topic':
					$topic = $this->smf->sql->query("
						SELECT	t.id_topic as id
						FROM {$db_prefix}topics t
						JOIN {$db_prefix}messages m
							ON m.id_msg = t.id_first_msg
						WHERE t.id_board = ?
						ORDER BY m.poster_time ASC
						LIMIT 0,1;
					",'i',$this->id)->assoc_result;
					if($topic){
						$topic = $this->smf->topic($topic['id']);
					}
					return $topic;
				break;
				case 'permissions':
					return $this->permissions();
				break;
				case 'group_ids':
					return explode($this->member_groups,',');
				break;
				case 'groups':
					$gs = $this->group_ids;
					$a = array();
					foreach($gs as $g){
						if($g > 0){
							array_push($a,$this->smf->group($g));
						}
					}
					return $a;
				break;
				case 'public':
					return in_array(0,$this->group_ids);
				break;
				case 'guest':
					return in_array(-1,$this->group_ids);
				break;
				case 'visible':
					return $this->visible();
				break;
			}
		}
		public function read($member=null){
			$member = is_null($member)?$this->smf->current_user:$member;
			$this->smf->sql->query("
				INSERT INTO {$db_prefix}log_mark_read (
					id_msg,
					id_member,
					id_board
				)
				VALUES (?,?,?)
				ON DUPLICATE KEY UPDATE
					id_msg = IF(VALUES(id_msg)>id_msg,VALUES(id_msg),id_msg);
			",'iii',
				$this->id,
				$member->id,
				$this->board->id
			)->execute();
		}
		public function permission($permission,$member=null){
			global $db_prefix;
			$member = is_null($member)?$this->smf->current_user:($member instanceof Member?$member:$this->smf->member($member));
			$groups = implode(',',$member->group_ids);
			$permission = is_array($permission)?$permission:array($permission);
			$permissions = implode(',',$this->smf->sql->escape($permission));
			$request = $this->smf->sql->query("
				SELECT MIN(bp.add_deny) AS add_deny
				FROM {$db_prefix}boards AS b
				INNER JOIN {$db_prefix}board_permissions AS bp
					ON (bp.id_profile = b.id_profile)
				LEFT JOIN {$db_prefix}moderators AS mods
					ON mods.id_board = b.id_board
					AND mods.id_member = ?
				WHERE b.id_board = ?
				AND bp.id_group IN (
					{$groups},
					?
				)
				AND bp.permission IN (
					{$permissions}
				)
				AND (
					mods.id_member IS NOT NULL
					OR bp.id_group != ?
				)
				GROUP BY b.id_board
			",'iiii',$member->id,3,$this->id,3)->assoc_results; // 3 == moderator apparently
			$ret = true;
			foreach($request as $row){
				$ret &= !empty($row['add_deny']);
			}
			return $ret;
		}
		public function permissions($member=null){
			global $db_prefix;
			$member = is_null($member)?$this->smf->current_user:($member instanceof Member?$member:$this->smf->member($member));
			$groups = implode(',',$member->group_ids);
			$permissions = $this->smf->sql->query("
				SELECT	bp.permission,
						bp.add_deny AS add_deny
				FROM {$db_prefix}boards AS b
				INNER JOIN {$db_prefix}board_permissions AS bp
					ON (bp.id_profile = b.id_profile)
				LEFT JOIN {$db_prefix}moderators AS mods
					ON mods.id_board = b.id_board
					AND mods.id_member = ?
				WHERE b.id_board = ?
				AND bp.id_group IN (
					{$groups},
					?
				)
				AND (
					mods.id_member IS NOT NULL
					OR bp.id_group != ?
				)
				GROUP BY bp.permission
			",'iiii',$member->id,3,$this->id,3)->assoc_results; // 3 == moderator apparently
			$ret = array();
			foreach($permissions as $permission){
				if(!empty($permission['add_deny'])){
					array_push($ret,$permission['permission']);
				}
			}
			return $ret;
		}
		public function visible($member=null){
			$member = is_null($member)?$this->smf->current_user:($member instanceof Member?$member:$this->smf->member($member));
			return $this->guest || $this->public || count(array_diff($this->group_ids,$member->group_ids))>0;
		}
	}
?>